package Hawat::Module::Whois;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Hawat::Module::Whois - Short module description

=head1 SYNOPSIS

 use Hawat::Module::Whois;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);
use File::Find;

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Hawat::Module;
use Whois::Service;
use Whois::Mongo;
use Whois::Negistry;
use Whois::Remote;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# Hash containing all valid commands supported by this module and their asociated
# callback names for indirect execution.
my %COMMANDS = (
            'search'    => [
                    '_command_search' ,
                    'Search through whois database',
                    '(STRING) what - what to search [MANDATORY]',
                ],
        );

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Hawat::Module');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item get_commands() [PUBLIC]

  Usage     :
  Purpose   : Return list of all commands this module provides
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on class
  Comments  :
  See Also  :

=cut

sub get_commands()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return %COMMANDS;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Hawat::Module::Whois instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    $self->{LOG_DIRS}  = ['/var/mentat/log/', '/opt/mentat/var/log/'];
    $self->{LOG_FILES} = {};
    $self->{WHOIS}     = Whois::Negistry->new('/var/mentat/whois-negistry.json',1);

    # Prepare various whois service objects
    $self->{WHOIS_LOCAL} = Whois::Service->new(
            Whois::Mongo->new('mentat', 'groups'),
        );
    $self->{WHOIS_NEGISTRY} = Whois::Negistry->new('/var/mentat/whois-negistry.json', 1, {
                    '/var/mentat/iplist-cesnet' => 'abuse@cesnet.cz',
                    '/var/mentat/iplist-pasnet' => 'abuse@cuni.cz',
                    '/var/mentat/iplist-grid'   => 'kouril@ics.muni.cz,michalp@ics.muni.cz',
                });
    $self->{WHOIS_REMOTE}   = Whois::Remote->new();

    return $self;
}

#-----------------------------------------------------------------------

# _command_search [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_search
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    my ($what,) = @args;
    die "Invalid usage: [$0 @$chain] must receive filter string as first argument\n" unless $what;

    my $start = time();

    $self->out(" - Searching for '$what' in whois database:\n");

    my (@results_local, @results_negistry, @results_remote, $type);
    # Query whois services - Reverse queries
    if ($what =~ /@/) {
        $type             = 'email';
        @results_local    = $self->{WHOIS_LOCAL}->rfind($what);
        @results_negistry = $self->{WHOIS_NEGISTRY}->rfind($what);
        @results_remote   = $self->{WHOIS_REMOTE}->rfind($what);
    }
    # Query whois services - Normal whois queries
    else {
        $type             = 'ip';
        @results_local    = $self->{WHOIS_LOCAL}->find($what);
        @results_negistry = $self->{WHOIS_NEGISTRY}->find($what);
        @results_remote   = $self->{WHOIS_REMOTE}->find($what);
    }

    # Prepare result
    $self->res->{'search'}          = $what;
    $self->res->{'result_local'}    = \@results_local;
    $self->res->{'result_negistry'} = \@results_negistry;
    $self->res->{'result_remote'}   = \@results_remote;

    $self->out("    * local results:\n");
    foreach my $r (@results_local) {
        $self->out("      - netname: ".$r->{netname}."\n");
        $self->out("        abuse:   ".join(', ', @{$r->{resolved_abuses}})."\n");
        $self->out("        network: ".$r->{network}."\n") if $r->{network};
    }
    $self->out("    * pure negistry results:\n");
    foreach my $r (@results_local) {
        $self->out("      - netname: ".$r->{netname}."\n");
        $self->out("        abuse:   ".join(', ', @{$r->{resolved_abuses}})."\n");
        $self->out("        network: ".$r->{network}."\n") if $r->{network};
    }
    $self->out("    * remote results:\n");
    foreach my $r (@results_local) {
        $self->out("      - netname: ".$r->{netname}."\n");
        $self->out("        abuse:   ".join(', ', @{$r->{resolved_abuses}})."\n");
        $self->out("        network: ".$r->{network}."\n") if $r->{network};
    }

    my $stop = time();
    $self->out(" - Viewing duration: " . ($stop - $start) . " [s]\n");
    $self->res->{'duration'} = $stop - $start;
}

=pod

=back

=cut

1;
