package Mentat::Message::Correlator::Context;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Message::Correlator::Context - Short module description

=head1 SYNOPSIS

 use Mentat::Message::Correlator::Context;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

Jan Mach
Cesnet, z.s.p.o
jan.mach@cesnet.cz
http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Message::Correlator::Context->new();
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Mentat::Message::Correlator::Context reference on success, croaks on failure
  Throws    : Dies, if not invoked on class
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    ### Creating Mentat.Message.Correlator.Context instance...
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item DESTROY() [PUBLIC]

  Usage     : This method must not be used directly
  Purpose   : Class destructor
  Arguments : None
  Returns   : Nothing
  Throws    : Dies, if not invoked on instance

=cut

sub DESTROY()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Destroying Mentat.Message.Correlator.Context instance...
}

#-------------------------------------------------------------------------------

=item id(;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    :
  Comments  :
  See Also  :

=cut

sub id(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($id,) = @_;

    $self->{ID} = $id if defined $id;
    return $self->{ID};
}

=item timeout(;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    :
  Comments  :
  See Also  :

=cut

sub timeout(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($timeout,) = @_;

    $self->{TIMEOUT} = $timeout if defined $timeout;
    return $self->{TIMEOUT};
}

=item message(;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    :
  Comments  :
  See Also  :

=cut

sub message(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;

    $self->{MESSAGE} = $message if defined $message;
    return $self->{MESSAGE};
}

=item update($) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    :
  Comments  :
  See Also  :

=cut

sub update($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;

    $self->{MESSAGE} = $message;
    $self->{COUNT}++;

    my $t = time;
    if ($self->{WINDOW}) {
        print STDERR "Updating timeout with sliding window\n";
        $self->{TIMEOUT} = $t + $self->{WINDOW};
    }
    if ($self->{TIMEOUT} > $self->{TTL}) {
        print STDERR "Updating timeout to TTL\n";
        $self->{TIMEOUT} = $self->{TTL};
    }

    return 1 if $self->condition_match();
    return 0;
}

=item condition_match($) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    :
  Comments  :
  See Also  :

=cut

sub condition_match($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($final,) = @_;

    # Not a final check
    unless ($final) {
        return 1 if ($self->{THRESH_MAX} and $self->{COUNT} >= $self->{THRESH_MAX});
        return 0;
    }

    # Final check
    return 1 if ($self->{THRESH_MAX} and $self->{COUNT} >= $self->{THRESH_MAX});
    return 1 if ($self->{THRESH_MIN} and $self->{COUNT} >= $self->{THRESH_MIN});
    return 1 if (not $self->{THRESH_MIN});
    return 0;
}


#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Message::Correlator::Context instance
#  Arguments : Unknown see the description in appropriate subclass
#  Returns   : Mentat::Message::Correlator::Context $self reference
#  Throws    : Dies, if not invoked on instance or given invalid arguments
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    ### Initializing Mentat.Message.Correlator.Context instance...
    my %config = @_;

    confess "Expecting context configurations" unless %config;
    confess "Context ID must be given as argument" unless $config{id};
    confess "Context maximum TTL must be given as argument" unless defined $config{ttl};
    confess "Context window must be smaller than TTL" if $config{window} and $config{window} > $config{ttl};
    confess "Threshold min must be smaller than threshold max" if $config{thresh_min} and $config{thresh_max} and $config{thresh_min} > $config{thresh_max};

    $self->{ID} = $config{id};

    my $t = time;
    $self->{TTL}     = $t + $config{ttl};
    $self->{WINDOW}  = $config{window};
    $self->{TIMEOUT} = (($self->{WINDOW})?($t + $self->{WINDOW}):$self->{TTL});

    $self->{COUNT}      = 0;
    $self->{THRESH_MIN} = $config{thresh_min};
    $self->{THRESH_MAX} = $config{thresh_max};

    $self->{MESSAGE} = undef;

    return $self;
}

=pod

=back

=cut

1;
